#---------------------------------------------------------------------------------------------#
# Title:  Empirical Substantive
# Purpose: Scale information
#---------------------------------------------------------------------------------------------#

## clear the global environment
rm(list=ls())

## set the working directory
path <- "E:\\Collapsing categories\\Syntax\\01empirical_substantive"

## load library
#install.packages("mirt")
#install.packages("nlme")
#install.packages("multcomp")
library(mirt) #Version 1.26.3
library("nlme")
library("multcomp")

## load data
data <- read.csv(paste0(path, "\\BULLY_G4_IDLINK.csv"), 
                 header = F, stringsAsFactors = F, na.strings = c(99))
data[,1] <- as.integer(gsub("﻿", "", data[,1]))

## rename variables and select use variables
names(data) <- c("IDCNTRY","IDSCHOOL","ID","IDSTUD","CSTRAT","JKREP","JKZONE",
                 "R09A","R09B","R09C","R09D","R09E","R09F",
                 "RM02A","RM02B","RM02C","RM02D","RM02E","RM03F",
                 "RM02A_M","RM02B_M","RM02C_M","RM02D_M","RM02E_M","RM03F_M",
                 "R04","R05A","R05B","R05C","R05D","R05E","R03","GIRL","R08A","R08C",
                 "CLWGT","STUWGT")
data <- data[,c("IDCNTRY", "IDSTUD", "R09A","R09B","R09C","R09D","R09E","R09F")]

## add country id
data$CNT <- dplyr::recode(data$IDCNTRY, 
                          '31'="AZE",'40'="AUT",'48'="BHR",'51'="ARM",'72'="BOT",
                          '152'="CHL",'158'="TWN",'191'="CRO",'203'="CZE",'208'="DNK",
                          '246'="FIN",'268'="GEO",'276'="DEU",'340'="HON",'344'="HKG",
                          '348'="HUN",'364'="IRN",'372'="IRL",'380'="ITA",'392'="JAP",
                          '398'="KZH",'410'="KOR",'414'="KUW",'440'="LIT",'470'="MTA",
                          '504'="MOR",'512'="OMN",'528'="NLD",'554'="NZL",'578'="NOR",
                          '616'="POL",'620'="PTG",'634'="QTR",'642'="ROM",'643'="RUS",
                          '682'="SAU",'688'="SER",'702'="SGP",'703'="SVK",'705'="SVN",
                          '724'="ESP",'752'="SWE",'764'="THA",'784'="UAE",'788'="TUN",
                          '792'="TUR",'840'="USA",'887'="YEM")

data <- data[, c("IDCNTRY","CNT","R09A","R09B","R09C","R09D","R09E","R09F")]

## collapse categories
# 4-CATEGORY
dataset0 <- dataset1 <- dataset2 <- dataset3 <- data 

# 3-CATEGORY: recode 0=0, 1=0, 2=1, 3=2
dataset1[dataset1==0] <-0
dataset1[dataset1==1] <-0
dataset1[dataset1==2] <-1
dataset1[dataset1==3] <-2

# 3-CATEGORY: recode 0=0, 1=1, 2=1, 3=2
dataset2[dataset2==0] <-0
dataset2[dataset2==1] <-1
dataset2[dataset2==2] <-1
dataset2[dataset2==3] <-2

# 3-CATEGORY: recode 0=0, 1=1, 2=2, 3=2
dataset3[dataset3==0] <-0
dataset3[dataset3==1] <-1
dataset3[dataset3==2] <-2
dataset3[dataset3==3] <-2

## combine four datasets
datasets <- list(dataset0, dataset1, dataset2, dataset3)

## create a space for saving TIF
CNTRY_TIF <- NULL 
Theta <- matrix(seq(-4,4,.001))

#---------------------------------------------------------------------------------------------#
# Fitting fit graded IRT 
#---------------------------------------------------------------------------------------------#
## areainfo: Compute the area within test or item information over a definite integral range.
## testinfo: Given an estimated model compute the test information.

for (dd in 1:length(datasets)){
  
  subdata <- datasets[[dd]] # select dataset for the specific condition
  IDCNTRY_list <- unique(data$IDCNTRY) # 48 countries
  
  for (i in 1:length(IDCNTRY_list)){ #calculate TIF for each country
    
    CNTRY_dataset <- subdata[subdata[,"IDCNTRY"]== IDCNTRY_list[i],]  #subset data for each country
    
    # information function under mirt package
    mirt_mod <- mirt( CNTRY_dataset[, c("R09A","R09B","R09C","R09D","R09E","R09F")], 
                      1, itemtype="graded",
                      technical = list(removeEmptyRows=TRUE))
    
    mirt_info <- areainfo(mirt_mod, c(-3,3)) 
    
    tinfo <- as.data.frame(cbind(testinfo(mirt_mod, Theta), Theta))
    names(tinfo) <- c("testinfo","theta")
    
    mirt_max_tinfo <- tinfo[order(tinfo$testinfo, decreasing = T),][1,1]
    mirt_max_theta <- tinfo[order(tinfo$testinfo, decreasing = T),][1,2]
    
    tif <- data.frame(IDCNTRY_list[i], 
                      nrow(CNTRY_dataset),  
                      mirt_info$TotalInfo, 
                      mirt_max_tinfo, 
                      mirt_max_theta,
                      dd)
    
    names(tif) <- c("IDCNTRY", 
                    "COUNT", 
                    "mirt_infoarea", 
                    "mirt_max_tinfo", 
                    "mirt_max_theta",
                    "cond")
    
    CNTRY_TIF <- rbind(CNTRY_TIF, tif)
    
  }
}


#---------------------------------------------------------------------------------------------#
# ANOVA for scale information 
#---------------------------------------------------------------------------------------------#

tif <- CNTRY_TIF

# 1. Area under the curve
#tifarea <- log(tif$mirt_infoarea)
tifarea <- (tif$mirt_infoarea)
# Add a Normal Curve 
h<-hist(tifarea, breaks=10, col="red", xlab="tifarea", 
        main="Histogram with Normal Curve") 
xfit<-seq(min(tifarea),max(tifarea),length=40) 
yfit<-dnorm(xfit,mean=mean(tifarea),sd=sd(tifarea))
yfit <- yfit*diff(h$mids[1:2])*length(tifarea) 
lines(xfit, yfit, col="blue", lwd=2) 

# Using the raw tifarea - they are a *little* bit closer to normal. 
y <- (tif$mirt_infoarea)
x <- factor(tif$Cond)
u <- factor(tif$IDCNTRY)

newdat <- data.frame(y, x, u)

# Using the log of SEs - they are a *little* bit closer to normal. 
aovtifarea <- lme(y ~ x, random = ~1|u/x, data = newdat)

summary(aovtifarea)

# Tukey's Honest Significant Differences
hsd_aovtifarea <- summary(glht(aovtifarea, linfct=mcp(x="Tukey")))
hsd_aovtifarea <- as.data.frame(summary(hsd_aovtifarea)$test[ c('coefficients', 'sigma', 'tstat', 'pvalues')] )
write.csv(hsd_aovtifarea , paste0(path, "\\hsd_aovtifarea.csv"), row.names = F)

# 2. Max info
#tifmax <- log(tif$mirt_max_tinfo)
tifmax <- (tif$mirt_max_tinfo)
# Add a Normal Curve 
h<-hist(tifarea, breaks=10, col="red", xlab="tifmax", 
        main="Histogram with Normal Curve") 
xfit<-seq(min(tifmax),max(tifmax),length=40) 
yfit<-dnorm(xfit,mean=mean(tifmax),sd=sd(tifmax))
yfit <- yfit*diff(h$mids[1:2])*length(tifmax) 
lines(xfit, yfit, col="blue", lwd=2) 

# Using the raw tifarea - they are a *little* bit closer to normal. 
y <- (tif$mirt_max_tinfo)
x <- factor(tif$Cond)
u <- factor(tif$IDCNTRY)

newdat <- data.frame(y, x, u)

# Using the log of SEs - they are a *little* bit closer to normal. 
aovtifmax <- lme(y ~ x, random = ~1|u/x, data = newdat)

summary(aovtifmax)

# Tukey's Honest Significant Differences
hsd_aovtifmax <- summary(glht(aovtifmax, linfct=mcp(x="Tukey")))
hsd_aovtifmax <- as.data.frame(summary(hsd_aovtifmax)$test[ c('coefficients', 'sigma', 'tstat', 'pvalues')] )
write.csv(hsd_aovtifmax , paste0(path, "\\hsd_aovtifmax.csv"), row.names = F)

# 3. Theta at max
#tiftheta <- log(tif$mirt_max_theta)
tiftheta <- (tif$mirt_max_theta)
# Add a Normal Curve 
h<-hist(tiftheta, breaks=10, col="red", xlab="tiftheta", 
        main="Histogram with Normal Curve") 
xfit<-seq(min(tiftheta),max(tiftheta),length=40) 
yfit<-dnorm(xfit,mean=mean(tiftheta),sd=sd(tiftheta))
yfit <- yfit*diff(h$mids[1:2])*length(tiftheta) 
lines(xfit, yfit, col="blue", lwd=2) 

# Using the raw tifarea - they are a *little* bit closer to normal. 
y <- (tif$mirt_max_theta)
x <- factor(tif$Cond)
u <- factor(tif$IDCNTRY)

newdat <- data.frame(y, x, u)

# Using the log of SEs - they are a *little* bit closer to normal. 
aovtiftheta <- lme(y ~ x, random = ~1|u/x, data = newdat)

summary(aovtiftheta)

# Tukey's Honest Significant Differences
hsd_aovtiftheta <- summary(glht(aovtiftheta, linfct=mcp(x="Tukey")))
hsd_aovtiftheta <- as.data.frame(summary(hsd_aovtiftheta)$test[ c('coefficients', 'sigma', 'tstat', 'pvalues')] )
write.csv(hsd_aovtiftheta , paste0(path, "\\hsd_aovtiftheta.csv"), row.names = F)
